#pragma once

/*  Copyright [2020] [Verdant Robotics]

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
*/

#include <NvVideoEncoder.h>
#include <cuda.h>

#include <cstdio>

struct Image {
  unsigned int pitch;
  unsigned int width;
  unsigned int height;
  unsigned long long cuda_handle;
};

namespace libvideo {

using size_t = std::size_t;

class NvMmEncoder {
  std::string name_;
  std::string filename_;
  size_t width_ = 0;
  size_t height_ = 0;
  size_t fps_ = 0;
public:
  ~NvMmEncoder() { deInit(); }

  bool init(const std::string_view encName, size_t width, size_t height, size_t fps,
            CUdevice cudaDevice = CU_DEVICE_INVALID) ;
  void deInit() ;

  bool beginEncoding(std::string_view filename, std::string_view message = {}) ;
  bool pushFrame(const Image& image) ;
  // bool pushFrame(NvBuffer& buffer);
  bool endEncoding() ;

  NvVideoEncoder* encoder() { return encoder_; }
  void* mp4Writer() { return mp4Writer_; }

  const std::string& name() const {return name_;}
  const std::string& filename() const {return filename_; }
  size_t width() const {return width_;}
  size_t height() const {return height_;}
  size_t fps() const {return fps_;}

private:
  size_t frameIndex_ = 0;
  bool running_ = false;

  std::FILE* outputFilePtr_ = nullptr;
  void* muxer_ = nullptr;
  CUcontext cudaContext_ = nullptr;
  CUdevice cudaDevice_ = CU_DEVICE_INVALID;

  NvVideoEncoder* encoder_ = nullptr;
  void* mp4Writer_ = nullptr;
};

}  // namespace libvideo
