#include <stdio.h>
#include <stdlib.h>
#include <cuda.h>
#include <cuda_runtime.h>
#include <string>

#include "nvmm_encoder.h"

constexpr unsigned int WIDTH = 1920;
constexpr unsigned int HEIGHT = 1080;
constexpr unsigned int FPS = 30;
constexpr unsigned int FRAMES_PER_VID = 120;
constexpr unsigned int NUM_VIDEOS = 3000;

#define CUDA_CALL(call)                                                                                                          \
    do                                                                                                                           \
    {                                                                                                                            \
        cudaError_t res = call;                                                                                                  \
        if (cudaSuccess != res)                                                                                                  \
        {                                                                                                                        \
                                                                                                                                 \
            printf("Image CUDA API failure %s: %s. Source: %s @ %s:%d\n", cudaGetErrorName(res), cudaGetErrorString(res), #call, \
                   __FILE__, __LINE__);                                                                                          \
        }                                                                                                                        \
    } while (0)

void SweepVertical(const size_t &row, const size_t &height, Image &img);

int main(int argc, char **argv)
{
    printf("Starting video encoding test\n");

    libvideo::NvMmEncoder encoder;
    Image img;
    CUdevice device = -2;
    // CUcontext context;
    // CUdeviceptr device_ptr;
    void *device_ptr;
    size_t pitch;
    size_t width_in_bytes = WIDTH;

    CUDA_CALL(cudaMallocPitch(&device_ptr, &pitch, width_in_bytes, HEIGHT + HEIGHT / 2));

    img.cuda_handle = reinterpret_cast<unsigned long long>(device_ptr);
    img.width = WIDTH;
    img.height = HEIGHT;
    img.pitch = pitch;

    // set the chroma planes
    CUdeviceptr chroma_planes_ptr = CUdeviceptr(reinterpret_cast<unsigned long long>(device_ptr)) + pitch * HEIGHT;
    CUDA_CALL(cudaMemset(reinterpret_cast<void *>(chroma_planes_ptr), 128, pitch * (HEIGHT / 2)));

    // set the luma plane
    CUDA_CALL(cudaMemset(device_ptr, 255, pitch * HEIGHT));

    constexpr size_t sweep_delta = HEIGHT / FPS;
    encoder.init("enc0", WIDTH, HEIGHT, FPS, device);
    std::string filename = "test-vid.mp4";

    for (size_t i = 0; i < NUM_VIDEOS; ++i)
    {
        std::stringstream ss;
        ss << std::to_string(i) << "-" << filename;
        encoder.beginEncoding(ss.str(), "test");
        for (unsigned int f = 0; f < FRAMES_PER_VID; f++)
        {
            SweepVertical(f * sweep_delta % HEIGHT, HEIGHT / 100, img);
            encoder.pushFrame(img);
        }
        encoder.endEncoding();
    }

    return 0;
}

void SweepVertical(const size_t &row, const size_t &height, Image &img)
{
    // reset previous
    CUDA_CALL(cudaMemset(reinterpret_cast<void *>(img.cuda_handle), 255, img.pitch * img.height));
    // set current
    CUdeviceptr data = CUdeviceptr(img.cuda_handle) + img.pitch * row;
    CUDA_CALL(cudaMemset(reinterpret_cast<void *>(data), 0, img.pitch * height));
}